#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------
# Copyright (c) Merchise Autrement [~º/~] and Contributors
# All rights reserved.
#
# This is free software; you can do what the LICENCE file allows you to.
#
{
    'name': 'test_one2one',
    'author': 'Merchise Autrement [~º/~]',
    'installable': True,
    'auto_install': False,
}
